# -*- coding: utf-8 -*-
#
# @author: <sylvain.herledan@oceandatalab.com>
# @date: 2016-11-02
#
# Copyright (C) 2016-2019 OceanDataLab
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
This module handles the serialization of data bucket objects
"""

import logging
import flatbuffers
import SEAScope.API.DataBucket
import SEAScope.types.data_info

logger = logging.getLogger(__name__)


def serialize(builder, bucket_obj):
    """
    Serialize a data bucket using FlatBuffers.

    Parameters
    ----------
    builder : flatbuffers.builder.Builder
        The FlatBuffers builder instance which serializes data. If this
        parameter is None, then a new builder will be created
    bucket_obj : :obj:`dict`
        Dictionary which contains information about the data bucket to
        serialize.
        It must have 2 keys:

        - ``info`` : a :obj:`dict` providing information about the data
          contained in the bucket. It must have all the keys required by
          :func:`SEAScope.types.data_info.serialize`

        - ``data`` : a :obj:`list` of `numpy.uint8` which contains values
          packed as unsigned bytes

    Returns
    -------
    tuple(flatbuffers.builder.Builder, int)
        A tuple which contains two elements:

        - the :obj:`flatbuffers.builder.Builder` instance which has been used
          to serialize data
        - an :obj:`int` which is the address/offset of the serialized object
          in the builder buffer
    """
    if builder is None:
        builder = flatbuffers.Builder(0)

    _, info = SEAScope.types.data_info.serialize(builder, bucket_obj['info'])

    cls = SEAScope.API.DataBucket

    buffer_len = len(bucket_obj['buffer'])
    cls.DataBucketStartBufferVector(builder, buffer_len)
    # Inefficient!
    # TODO: optimize this later, using internal methods of flatbuffers
    for value in bucket_obj['buffer'][::-1]:
        builder.PrependUint8(value)
    data_buffer = builder.EndVector(buffer_len)

    cls.DataBucketStart(builder)
    cls.DataBucketAddInfo(builder, info)
    cls.DataBucketAddBuffer(builder, data_buffer)
    bucket = cls.DataBucketEnd(builder)

    return builder, bucket


def deserialize(o):
    """Not implemented"""
    pass
